# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class StrataDeleteServersResponse(UniversalBaseModel):
    deleted_servers: typing_extensions.Annotated[typing.List[str], FieldMetadata(alias="deletedServers")] = (
        pydantic.Field()
    )
    """
    List of Klavis MCP server names that were deleted
    """

    deleted_external_servers: typing_extensions.Annotated[
        typing.List[str], FieldMetadata(alias="deletedExternalServers")
    ] = pydantic.Field()
    """
    List of external MCP server names that were deleted
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
