# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class Environment(str, enum.Enum):
    """
    QuickBooks environment to authorize ('sandbox' or 'production')
    """

    SANDBOX = "sandbox"
    PRODUCTION = "production"

    def visit(self, sandbox: typing.Callable[[], T_Result], production: typing.Callable[[], T_Result]) -> T_Result:
        if self is Environment.SANDBOX:
            return sandbox()
        if self is Environment.PRODUCTION:
            return production()
